
# q-expansion of newform 510.2.w.b, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 510
weight = 2
poly_data = [1, 0, 0, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[341, [-1, 0, 0, 0]], [307, [0, 0, -1, 0]], [241, [0, 1, 0, 0]]]
aps_data = [[1, 0, 0, 0], [-1, 1, 1, 0], [-2, 0, -1, 0], [-2, -2, -1, 1], [-3, 0, 0, -3], [-3, 0, 3, 2], [-3, -2, 0, -2], [-1, 6, -1, 0], [-2, 2, 1, 1], [-2, 2, -1, -1], [0, 0, -1, 1], [2, -1, 1, -2], [-1, 4, -4, 1], [0, -4, -6, -4], [-3, 0, 3, 4], [0, -2, 4, -2], [-5, -2, -5, 0], [6, -1, -1, 6], [-3, 0, 3, 2], [4, -4, 5, 5], [-3, -6, 6, 3], [-8, -5, -5, -8], [0, -4, -6, -4], [0, 10, 0, 10], [-5, 0, 0, -5], [0, 2, 8, 2], [-7, 8, -7, 0], [-3, -2, 2, 3], [-6, 5, -5, 6], [5, -5, 4, 4], [4, -4, 0, 4], [-5, -5, 0, 0], [-1, 12, -1, 0], [1, -1, -4, -4], [0, -10, -8, -10], [1, 0, -1, -8], [9, 10, 9, 0], [-3, 3, -6, -6], [-12, 3, 3, -12], [-6, -7, 7, 6], [-3, 6, -3, 0], [-6, 9, -9, 6], [-12, -4, 0, 4], [9, 9, -10, 10], [6, -6, 1, 1], [0, 0, 11, -11], [9, -6, -6, 9], [-12, -8, 0, 8], [1, 1, 4, -4], [-13, 0, 13, -2], [1, 4, 4, 1], [-12, 0, 0, 0], [-5, -5, 14, -14], [12, -2, 0, 2], [6, -2, 0, 2], [-8, 0, 0, 0], [-10, -10, -17, 17], [-12, 8, 0, -8], [-8, 19, 19, -8], [-11, 0, 11, -12], [9, 9, 2, -2], [7, -2, 7, 0], [-9, 0, 9, -18], [16, 16, -3, 3], [7, -7, -8, -8], [-16, -16, 11, -11], [-3, -2, -3, 0], [-1, 12, -12, 1], [7, -7, 14, 14], [-5, 0, 5, -6], [-11, -8, -11, 0], [3, 0, -3, -8], [-2, -2, -1, 1], [-7, -6, -7, 0], [-15, -15, -12, 12], [16, 8, 0, -8], [1, 0, -1, -2], [-12, 12, -11, -11], [-3, 10, -10, 3], [0, 2, 8, 2], [-3, -12, 12, 3], [0, -10, -24, -10], [24, 9, 9, 24], [0, 12, 12, 12], [-2, 2, 13, 13], [15, 0, -15, -10], [-5, 8, 8, -5], [0, 4, 4, 4], [-7, -18, -7, 0], [11, 0, -11, 4], [0, -6, -18, -6], [14, 14, -5, 5], [0, 0, 5, 5], [17, 2, 17, 0], [1, 18, 18, 1], [12, 12, 3, -3], [30, 4, 0, -4], [9, 10, 10, 9], [-7, 0, 7, 2], [-16, -16, -13, 13], [21, -12, 12, -21], [-1, 0, 1, 14], [0, 12, -22, 12], [3, 8, 3, 0], [-5, -4, -4, -5], [5, 0, -5, 24], [0, -2, -34, -2], [0, -10, -8, -10], [0, -12, 2, -12], [7, -18, 18, -7], [-8, 5, 5, -8], [-7, -10, -7, 0], [-27, 8, -8, 27], [-3, -22, 22, 3], [-23, 0, 23, -4], [-1, -1, 8, -8], [13, -20, -20, 13], [-21, 8, -21, 0], [18, 18, -7, 7], [0, -6, 26, -6], [-5, 0, 5, 2], [-7, 7, -2, -2], [-8, 5, 5, -8], [-27, 6, -6, 27], [-15, 14, -14, 15], [-18, 4, 0, -4], [-4, -4, -5, 5], [8, -8, 25, 25], [-15, -8, -15, 0], [2, 32, 0, -32], [21, 0, -21, 10], [-4, -17, -17, -4], [-10, -10, 17, -17], [2, 8, 0, -8], [-10, -12, 0, 12], [-26, -16, 0, 16], [-46, 0, 0, 0], [9, -16, -16, 9], [-14, 4, 0, -4], [11, 12, 12, 11], [11, -11, 2, 2], [8, 8, 25, -25], [10, -10, -21, -21], [11, 11, 2, -2], [-26, 4, 0, -4], [-26, 9, -9, 26], [-18, 13, 13, -18], [5, -5, 28, 28], [-5, 0, 5, 18], [15, -12, 15, 0], [-16, -16, -19, 19], [19, -19, 4, 4], [-9, -14, -9, 0], [-20, 11, 11, -20], [-35, 35, 10, 10], [-8, 11, -11, 8], [0, 8, -30, 8], [-11, -6, 6, 11], [0, -4, -12, -4], [4, 3, 3, 4], [5, -6, 6, -5], [1, 0, -1, -40], [0, -24, 10, -24], [-5, 18, -5, 0], [0, 18, -32, 18], [20, 5, -5, -20], [2, -2, 9, 9], [4, -4, -31, -31]]
